/*
	MMC/SD Card Hardware Interface Mudule for DOS/V Parallel port

	Copyright(C) 2002 Tatsuyuki Satoh
*/
/* #include <stdio.h> */
#include "mmc_if.h"

/*
	Hardware Interface Design

	DOS/V Printer Port

	PD0 : MMC.5(CLK) 3.3V drive
	      H = High (MMC.4(VDD)==3.3V)
	      L = Low  (0V)
	PD1 : MMC.1(CD/DAT3) 3.3V drive
	      H = High (MMC.4(VDD)==3.3V)
	      L = Low  level (0V)
	PD2 : MMC.2(CMD) open drain drive
	      H = Hi-Z (100Kohm Pull-up to MMC.4(VDD))
	      L = Low  level (0V)
	PD3-7 : MMC/SD Power signal
	      H = Power On
	      L = Power Off
	BUSY: MMC.7(DAT0) input signal

	* An OpenDrain driver and a level converter are necessary. 
	* MMC.2 and MMC.7 should be pull-up with 50-100Kohm to MMC.4(VDD)
	* If a SD card is used, SD.9, SD.8 are pull-up in the same way, too. 
	* If OS is WinNT/2K/XP , GIVEIO service is necessary for the Direct I/O access. 

*/
#define MMC_O_PORT 0x378 /* port address of output signal */
#define MMC_O_CLK  0x01 /* 2:D0 */
#define MMC_O_CS   0x02 /* 3:D1 */
#define MMC_O_CMD  0x04 /* 4:D2 */
#define MMC_O_PON  0xf8 /* default value of outport on power up   */
#define MMC_O_POF  0x00 /* default value of outport on power down */
#define MMC_O_END  0x00 /* */

#define MMC_I_PORT (MMC_O_PORT+1) /* port address of input signal */
#define MMC_I_DAT  0x80 /* 11:BUSY */

/* macro of output/input MMC port */
#define MMC_OUT(VAL) _outp(MMC_O_PORT,(VAL))
#define MMC_IN() (_inp(MMC_I_PORT)^0x80)

unsigned char mmc_o_val;

/* Start MMC interface port */
int mmc_hw_start(void)
{
	mmc_hw_power(0);
	return 1;
}

/* Stop MMC interface port */
int mmc_hw_stop(void)
{
	/* paralell port init */
	MMC_OUT(MMC_O_END);
}

/* MMC power controll */
void mmc_hw_power(unsigned char power_on)
{
	mmc_o_val = power_on ? MMC_O_PON|MMC_O_CS|MMC_O_CMD  : MMC_O_POF|MMC_O_CS|MMC_O_CMD;
	MMC_OUT(mmc_o_val);
}

/* MMC CS controll */
void mmc_hw_cs(unsigned char cs)
{
	if(cs)
		mmc_o_val |= MMC_O_CS;
	else
		mmc_o_val &= ~MMC_O_CS;
	MMC_OUT(mmc_o_val);
}

/* Serial output one byte */
void mmc_serial_out(unsigned char val)
{
	unsigned char i;

	mmc_o_val |= MMC_O_CLK;
	/* MSB first */
	for(i=0x80;i;i>>=1)
	{
		if(val&i)
			mmc_o_val |= MMC_O_CMD;
		else
			mmc_o_val &= ~MMC_O_CMD;

		MMC_OUT(mmc_o_val & ~MMC_O_CLK);
		MMC_OUT(mmc_o_val);
	}
/*	printf("SOUT %02X:",val);*/
}

/* Serial input one byte */
unsigned char mmc_serial_in(void)
{
	unsigned char i;
	unsigned char in_val = 0;

	mmc_o_val |= MMC_O_CMD | MMC_O_CLK;

	/* MSB first */
	for(i=0x80;i;i>>=1)
	{
		MMC_OUT(mmc_o_val&~MMC_O_CLK);
		MMC_OUT(mmc_o_val&~MMC_O_CLK);
		MMC_OUT(mmc_o_val);
		if(MMC_IN() & MMC_I_DAT)
			in_val |= i;
	}
/*	printf("SIN %02X:",in_val);*/
	return in_val;
}
