/*
	MMC/SD Card read / write test sample

	Copyright(C) 2002 Tatsuyuki Satoh

	Carry out the following command to see sector DUMP. 
	"mode CON lines=50"
*/
#include <stdio.h>
#include "giveio.h"
#include "mmc_spi.h"

/* sector buffer */
unsigned char buff[512];

/* GIVEIO install/start result */
int giveio_result;

/* main */
int main()
{
	long sec;
	int i;
	int cmd;
	int res;

	printf("MMC/SD Card SPI Access Tester\n");
	printf("Copyright(C) 2002 Tatsuyuki Satoh\n\n");

	/* GIVEIO open */
	printf("Open GIVEIO for WinNT/2K/XP : ");
	giveio_result = giveio_start();
	switch(giveio_result)
	{
	case GIVEIO_INSTALL:
		printf("Installed: ");
	case GIVEIO_START:
		printf("Started: ");
	case GIVEIO_OPEN:
		printf("Opened\n");
		break;
	case GIVEIO_WIN95:
		printf("Win95/98/Me\n");
		break;
	case GIVEIO_ERROR:
		printf("Can't Access Direct I/O\n");
		return 0;
	}

	/* MMC initialise */
	printf("Initialize MMC Card for SPI mode:");
	res = mmc_init_spi();
	if(res!=MMC_NO_ERR)
	{
		switch(res)
		{
		case MMC_ERR_HW:
			printf("MMC hardware error\n");
			break;
		case MMC_ERR_CMD0:
			printf("CMD0 response error\n");
			break;
		case MMC_ERR_CMD1:
			printf("CMD1 response error\n");
			break;
		default:
			printf("Error\n");
		}
	}
	else
	{
		printf("OK\n");
#if 0
		/* Write Sector 0x00000001 with test data */
		sec = 1; /* write sector */
		for(i=0;i<512;i++)
			buff[i] = i%256;
		printf("Write Sector %06XH :",sec);
		if( spi_write(sec,buff) == MMC_NO_ERR )
			printf("OK\n");
		else
			printf("Error\n");
#endif

		/* read & dump sector */
		sec = -1;
		while(1)
		{
			printf("Read Sector?(CR=NEXT,SPC=ADDRESS,E=END)");
			switch( getch() )
			{
			case 0x1b: /* ESC */
			case 'E':
			case 'e':
				sec = -1; break;
			case 0x20:
				printf("\nInput Sector Number(HEX)?");
				scanf("%lx",&sec);
				break;
			default:
				sec++;
			}

			/* end? */
			if(sec==-1)
				break;

			printf("\nRead Sector %06XH :",sec);
			if( spi_read(sec,buff) != MMC_NO_ERR )
				printf("Error\n");
			else
			{
				printf("OK");
				/* DUMP */
				for(i=0;i<512;i++)
				{
					if((i%16)==0)
						printf("\n%03X=",i);
					printf("%02X ",buff[i]);
				}
				printf("\n");
			}
		}
		/* MMC finish */
		mmc_disable();
	}

	printf("\n");

	/* GIVEIO CLOSE */
#if 0
	/* Always Remove Driver */
	giveio_result = GIVEIO_REMOE;
#endif
	printf("GIVEIO close: ");
	switch(giveio_result)
	{
	case GIVEIO_REMOVE:
		printf("Remove :");
	case GIVEIO_STOP:
		printf("Stop :");
	}
	printf("\n");
	giveio_result = giveio_stop(giveio_result);
	switch(giveio_result)
	{
	case GIVEIO_REMOVE:
		printf("Can't remove GIVEIO\n");
		break;
	case GIVEIO_STOP:
		printf("Can't stop GIVEIO\n");
		break;
	}

}
